//
//	ADComment.m
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "ADComment.h"
#import <Foundation/NSArray.h>
#import <MiscFoundation.subproj/NSString+MiscRegex.h>
#import <MiscFoundation.subproj/NSString+MiscAdditions.h>
#import "ADUtilities.h"

@implementation ADComment : NSObject

+ commentWithString:(NSString *)aString
{
	return [[[self alloc] initWithString:aString] autorelease];
}

+ commentWithNormalString:(NSString *)aString
{
	return [[[self alloc] initWithNormalString:aString] autorelease];
}

/*"
 * The designated initializer for ADComment; aString should have the raw
 * comment (including the comment-start (/*") and comment-end characters).
 * aString is stored in the rawString ivar.  The commentString ivar
 * contains just the actual string of the comment -- i.e. aString with
 * leading spaces and *'s removed, and whitespace squashed except for
 * preserving double line breaks as paragraphs.
"*/
- initWithString:(NSString *)aString
{
	NSRange frontQuoteRange = [aString rangeOfString:@"\""];
	NSRange backQuoteRange = [aString rangeOfString:@"\"" options:NSBackwardsSearch];
	NSRange commentRange;
	
	[super init];

	commentRange.location = NSMaxRange(frontQuoteRange);
	commentRange.length = backQuoteRange.location - commentRange.location;
	
    /*
     * Trim out excess stuff -- leading spaces and *'s, and squash spaces
     * while preserving double line breaks as paragraphs.
     */
	commentString = [aString substringWithRange:commentRange];
	commentString = [commentString stringByTrimmingWhitespace];
//	commentString = [commentString stringByReplacingEveryOccurrenceOfRegex:@"^[ \t*]+"
//								   withString:@""];
	commentString = [commentString stringByReplacingEveryOccurrenceOfRegex:@"^[ \t]*[*]*[ \t]*"
								   withString:@""];
	commentString = [commentString stringByReplacingEveryOccurrenceOfRegex:
								   @"([^ \t\n])[ \t]+([^ \t\n])"
								   withString:@"$1 $2"
								   options:MiscUseMatchSubstitutions];
	commentString = [commentString stringByReplacingEveryOccurrenceOfRegex:
								   @"([^ \t\n])[ \t]*\n[ \t]*([^ \t\n])"
								   withString:@"$1 $2"
								   options:MiscUseMatchSubstitutions];
	commentString = [commentString stringByReplacingEveryOccurrenceOfRegex:
								   @"([^ \t\n])[ \t]*\n[ \t]*\n[ \t\n]*([^ \t\n])"
								   withString:@"$1\n\n$2"
								   options:MiscUseMatchSubstitutions];
	
	[commentString retain];
	rawString = [aString retain];

	return self;
}

/*"
 * Initializer for an ADComment with a normal string; aString is simply
 * stored in the rawString and commentString ivars.
"*/
- initWithNormalString:(NSString *)aString
{
	rawString = [aString retain];
	commentString = [aString retain];
	return self;
}

- (NSArray *)italicizedWordArray
{
	return italicizedWordArray;
}

- (void)setItalicizedWordArray:(NSArray *)anArray
{
	SETIVAR(italicizedWordArray, anArray);
}

- (NSString *)rawCommentString
{
	return rawString;
}

- (NSString *)commentString
{
	return commentString;
}

/*"
 * Returns the commentString passed through ADFormatString().  This is the
 * preferred method to use in template files.
"*/
- (NSString *)formattedCommentString
{
	return ADFormatString(commentString, italicizedWordArray);
}

- description
{
	return [NSString stringWithFormat:@"/* %@ */", [self commentString]];
}

@end
