//
//	ADCompoundIvar.m
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "ADCompoundIvar.h"
#import <Foundation/NSCharacterSet.h>
#import <Foundation/NSArray.h>
#import "ADUtilities.h"

#if defined(NX_CURRENT_COMPILER_RELEASE) && (NX_CURRENT_COMPILER_RELEASE <= 330)
#define makeObjectsPerformSelector makeObjectsPerform
#endif

@implementation ADCompoundIvar

/*" Represents a "compound" ivar, i.e. a struct or a union. "*/

- (void)dealloc
{
	[subIvars release];
	[super dealloc];
}

/*" Returns the array of subivars that make up the structure or union. "*/
- (NSArray *)subIvars
{
	return subIvars;
}

/*" Sets the subivars array.  This method is used by the parser. "*/
- (void)setSubIvars:(NSArray *)ivarArray
{
	[ivarArray makeObjectsPerformSelector:@selector(setParentIvar:) withObject:self];
	SETIVAR(subIvars, ivarArray);
}

/*"
 * Sets the receiver's type.  This method is not provided by superclass, so
 * we need to implement it here.
"*/
- (void)setType:(NSString *)aType
{
	SETIVAR(type, aType);
}

/*" Returns a description of the ivar. "*/
- description
{
	return [NSString stringWithFormat:@" %@ {\n%@ } %@;", type, subIvars, name];
}

@end
