//
//	ADEntity.m
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "ADEntity.h"
#import "ADComment.h"
#import "ADUtilities.h"

@implementation ADEntity

/*"
 * ADEntity is the abstract superclass for all elements that may appear in
 * a file.
"*/

- (void)dealloc
{
	[entityDescription release];
	[super dealloc];
}

/*"
 * Sets the ADComment that describes the receiver.  Mainly for use by the
 * bison parser; if you want to set your own description
 * -#setDescriptionString: is probably a nicer interface.
"*/
- (void)setEntityDescription:(ADComment *)comment
{
	SETIVAR(entityDescription, comment);
	[comment setItalicizedWordArray:[self wordsToItalicize]];
}

/*"
 * Returns the ADComment parsed from the documentation that describes the
 * receiver.
"*/
- (ADComment *)entityDescription
{
	return entityDescription;
}

/*"
 * Sets the receiver's entityDescription to be an ADComment containing
 * aString (uses ADComment's +#commentWithNormalString method).
"*/
- (void)setDescriptionString:(NSString *)aString
{
	[self setEntityDescription:[ADComment commentWithNormalString:aString]];
}

/*"
 * Returns an array of words that should be italicized for this entity when
 * generating documentation -- the returned array passed to the receiver's
 * entityDescription during -#setEntityDescription.  By default returns
 * nil, but subclasses can override.
"*/
- (NSArray *)wordsToItalicize
{
	return nil;
}

@end
