//
//	ADFile.h
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <Foundation/NSObject.h>

@class NSArray, NSMutableArray, NSString;
@class ADFunction, ADGlobalVariable, ADConstant, ADMacro, ADTypedef;
@class ADInterface, ADClass, ADProtocol, ADCategory;

@interface ADFile : NSObject
{
	NSMutableArray	*functionArray;
	NSMutableArray	*variableArray;
	NSMutableArray	*constantArray;
	NSMutableArray	*macroArray;
	NSMutableArray	*typedefArray;
	NSMutableArray	*classArray;
	NSMutableArray	*categoryArray;
	NSMutableArray	*protocolArray;
	ADInterface		*mainInterface;
	NSString		*version;
	NSString		*copyrightYear;
	NSString		*copyrightOwner;
	NSString		*headerFilename;
	NSString		*sourceFilename;
	NSString		*importString;
}

/*" Initializing from a file "*/
- initWithHeaderFilename:(NSString *)header sourceFilename:(NSString *)source;

/*" Adding different types of entities "*/
- (void)addFunction:(ADFunction *)aFunction;
- (void)addVariable:(ADGlobalVariable *)aVariable;
- (void)addConstant:(ADConstant *)aConstant;
- (void)addMacro:(ADMacro *)aMacro;
- (void)addTypedef:(ADTypedef *)aTypedef;
- (void)addClass:(ADClass *)aClass;
- (void)addProtocol:(ADProtocol *)aProtocol;
- (void)addCategory:(ADCategory *)aCategory;
- (void)addInterface:(ADInterface *)interface;

/*" Finding entities "*/
- (ADClass *)classNamed:(NSString *)className;
- (ADProtocol *)protocolNamed:(NSString *)protocolName;
- (ADCategory *)categoryNamed:(NSString *)categoryName forClass:(NSString *)className;

/*" Getting attributes "*/
- (NSString *)version;
- (NSString *)copyrightYear;
- (NSString *)copyrightOwner;
- (NSString *)headerFilename;
- (NSString *)sourceFilename;
- (NSString *)importString;
- mainInterface;
- (BOOL)hasMainInterface;
- (NSString *)mainTitle;
- (NSArray *)functionArray;
- (NSArray *)variableArray;
- (NSArray *)constantArray;
- (NSArray *)macroArray;
- (NSArray *)typedefArray;

/*" Setting attributes "*/
- (void)setVersion:(NSString *)aString;
- (void)setCopyrightYear:(NSString *)aString;
- (void)setCopyrightOwner:(NSString *)aString;
- (void)setHeaderFilename:(NSString *)aString;
- (void)setSourceFilename:(NSString *)aString;
- (void)setImportString:(NSString *)aString;

/*" Parser support "*/
- (void)setCopyrightFromString:(NSString *)copyrightString;
- (void)setImportFromString:(NSString *)aString;

/*" Getting groupings of header entities "*/
- (NSArray *)groupArrayFromEntityArray:(NSArray *)entityArray;
- (NSArray *)functionGroupArray;
- (NSArray *)variableGroupArray;
- (NSArray *)constantGroupArray;
- (NSArray *)macroGroupArray;
- (NSArray *)typedefGroupArray;

/*" Getting array of subfiles "*/
- (NSArray *)subfileArray;

@end

