//
//	ADGlobalVariable.m
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "ADGlobalVariable.h"
#import <MiscFoundation.subproj/NSString+MiscAdditions.h>
#import "ADUtilities.h"

@implementation ADGlobalVariable
/*" Represents a C global variable "*/

+ variableWithString:(NSString *)string
{
	return [[[self alloc] initWithString:string] autorelease];
}

/*"
 * Takes string, which should include the entire "extern" definition, and
 * parses out the type and name of the variable.  The type does not include
 * the "extern" word.
"*/
- initWithString:(NSString *)string
{
	NSRange				  nameRange;
	NSRange				  externRange;

	[super init];

	nameRange = ADRangeOfVariableBeforeIndex(string, [string length]);	
	externRange = [string rangeOfString:[string wordNum:0]];

	name = [string substringWithRange:nameRange];

	type = [string substringToIndex:nameRange.location];
	type = [type substringFromIndex:NSMaxRange(externRange)];
	type = ADSquashTypeString(type);

	[name retain];
	[type retain];

	return self;
}

- (void)dealloc
{
	[name release];
	[type release];
	[super dealloc];
}
	
- (NSString *)name
{
	return name;
}

- (NSString *)type
{
	return type;
}

/*" Returns a description of the ivar. "*/
- description
{
	return [NSString stringWithFormat:@"extern %@%@;", type, name];
}


@end
