%{
//
//	SourceGrammar.y
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

	#import "ADClass.h"
	#import "ADComment.h"
	#import "ADProtocol.h"
	#import "ADIvar.h"
	#import "ADCategory.h"
	#import "ADMethod.h"
	#import "ADFile.h"

	#import <Foundation/NSDictionary.h>
	#import <Foundation/NSPathUtilities.h>
	#import <Foundation/NSUtilities.h>
	#import <stdio.h>
	#import <stdlib.h>
	#import <string.h>
	
	#undef yylex
	#define yylex SourceTokenizerlex

	/* Define some strings, since we can't use constant strings inside bison rules */
	#define EMPTYSTRING @""
	#define MDOCSTRING @"mdoc"	
	
	int yyerror(char *s);
	int yylex();

	extern int SourceTokenizerlineno;
	
	#define YYDEBUG 1
	#define YYERROR_VERBOSE 1

//	YYPARSE_PARAM doesn't work because of a snafu in bison.simple
//	#define YYPARSE_PARAM fileContents
//	#define ADFILE (ADFile *)fileContents
	static ADFile *globalFileContents;
	#define ADFILE (ADFile *)globalFileContents
%}

%expect 2

%token IMPLEMENTATION
%token END
%token SQUIGGLEBLOCK
%token <stringVal> IDENT
%token <stringVal> AUTODOCCOMMENT
%token <stringVal> STATICTYPE
%token <stringVal> IMPORTSTRING
%token <charVal> METHODSTART

%type <method>method
%type <method>arguments
%type <stringVal>argument
%type <mutableArray>manymethods
%type <mutableArray>vardecls
%type <comment>maybecomment
%type <stringVal>statictype
%type <object>implstart

%union
{
	NSString *stringVal;
	int		intVal;
	char	charVal;
	NSMutableArray *mutableArray;
	id	 object;
	ADMethod *method;
	ADComment *comment;
}

%%

file:	/* Empty */
		| file filepiece
		;

filepiece:  implstart maybecomment manymethods END
			{
				if ($2) [$1 setEntityDescription:$2];
				[$1 setMethodDescriptionsFromArray:$3];
			}
			| IMPORTSTRING
			{
				[ADFILE setImportFromString:$1];
			}
			;

implstart:  IMPLEMENTATION IDENT ':' IDENT vardecls
			{
				$$ = [ADFILE classNamed:$2];
			}
			| IMPLEMENTATION IDENT vardecls
			{
				if ([[[ADFILE sourceFilename] pathExtension] isEqual:MDOCSTRING])
					$$ = [ADFILE protocolNamed:$2];
				else
					$$ = [ADFILE classNamed:$2];

				// This would find it either way...
				// $$ = [ADFILE classNamed:$2];
				// if ($$ == nil) $$ = [ADFILE protocolNamed:$2];
			}
			| IMPLEMENTATION IDENT '(' IDENT ')'
			{
				$$ = [ADFILE categoryNamed:$4 forClass:$2];
			}
			;

vardecls:	/* Empty */       { $$ = nil; }
			| SQUIGGLEBLOCK { $$ = nil; }
			;


maybecomment: {$$ = nil; } | AUTODOCCOMMENT { $$ = [ADComment commentWithString:$1]; }
			;
			
manymethods:	/* Empty */       { $$ = [NSMutableArray array]; }
			| manymethods method  { $$ = $1; if ($2) [$$ addObject:$2]; }
			;

method:		maybecomment METHODSTART statictype IDENT arguments maybesemicolon
			maybecomment methodimp
			{
				$$ = $5;
				[$$ setClassMethod:($2 == '+')];
				[$$ setFirstArgumentName:$4];

				/* A comment before the method block will win if both are present */
				if ($7) [$$ setEntityDescription:$7];
				if ($1) [$$ setEntityDescription:$1];
			}
			| error {$$ = nil; }

methodimp:	/* Empty -- should only happen in a protocol's mdoc file */
			| SQUIGGLEBLOCK
			;

maybesemicolon: /* Empty */
			| ';'
			;

statictype:	/* Empty */		{$$ = EMPTYSTRING; }
			| STATICTYPE	{$$ = $1; }
			;

arguments:	/* Empty */
				{
					$$ = [ADMethod method];
				}
			| arguments argument
				{
					$$ = $1;
					[$$ addArgument:$2 type:EMPTYSTRING varName:EMPTYSTRING];
				}
;

	/* Just need to know the argument name part */
argument:  IDENT ':' STATICTYPE IDENT	{	$$ = $1;			}
		 | ':' STATICTYPE IDENT			{	$$ = EMPTYSTRING;	}
		 | ':' IDENT					{	$$ = EMPTYSTRING;	}
		 | IDENT ':' IDENT				{	$$ = $1;			}
		 ;

%%

extern void ADSetSourceFile(FILE *file);

void ADParseSourceFile(ADFile *fileContents)
{
	FILE *sourceFile;
	
	//yydebug = 1;
	if ([fileContents sourceFilename])
	{
		sourceFile = fopen([[fileContents sourceFilename] cString], "r");
		ADSetSourceFile(sourceFile);
		globalFileContents = fileContents;
		yyparse();
		fclose(sourceFile);
	}
}

int yyerror(char *s)
{
    // Having a function alongside methods in the class implementation
    // causes the grammar to fail (though it catches it with the error
    // rule). I haven't figured out a way to handle it more gracefully, so
    // for now I'm just not logging the error string...
	/* 	fprintf(stderr, "Warning, couldn't parse construct: %s\n", s); */
	return 1;
}

