/*
//	SourceTokenizer.l
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
*/	

%{

#import <Foundation/NSArray.h>
#import <Foundation/NSData.h>
#import <MiscFoundation.subproj/NSString+MiscRegex.h>

#import "ADClass.h"
#import "ADComment.h"
#import "ADProtocol.h"
#import "ADIvar.h"
#import "ADCategory.h"
#import "ADMethod.h"

#define YYSTRING [NSString stringWithCString:yytext length:yyleng]

#import "SourceGrammar.m.h"

#define yylval SourceGrammarlval

static int numSquiggles = 0;

// I really don't use the reject rule
#ifdef YY_USES_REJECT
#undef YY_USES_REJECT
#endif

// Uncomment this stuff if using the global yylloc variable.  Uncomment
// everything but the extern if passing yylloc as an argument to yylex.
//extern YYLTYPE yylloc;
//void ResetYYLoc(YYLTYPE *yyLocation, char *text, int textLength);
//void SetupYYLoc(YYLTYPE *yyLocation);
//#define YY_USER_ACTION ResetYYLoc(&yylloc, yytext, yyleng);
//#define YY_USER_INIT SetupYYLoc(&yylloc);

%}

%option noyywrap
%option yylineno

ident				[a-zA-Z_][a-zA-Z0-9_]*
ccomment			("/*"([^*]|"*"+[^*/])*"*"+"/")
objccomment			("//"[^\n]*\n?)
conststring         ["](\\.|[^\\"])*["]
whitespace			[ \t\r\n]*
statictype			(\([^:;@+{}-]*\))
methodstart			^{whitespace}[+-]{whitespace}{statictype}?
methodargument		":"{whitespace}{statictype}?{whitespace}{ident}


%s CLASS
%s METHOD
%x SQUIGGLES

%%

{ccomment}				{
							if (YYSTATE != INITIAL)
							{
								NSString *commentString = YYSTRING;
		
								/* Autodoc actually allows comments like '/****"  */
								if ([commentString containsRegex:@"\\`/[*]+\""] &&
									[commentString containsRegex:@"\"[*]+/\\'"])
								{
									yylval.stringVal = commentString;
									return AUTODOCCOMMENT;
								
									/* Warning avoidance */
									unput(' ');			// always needed
									goto find_rule;		// needed if %option yylineno
								}
							}
						}
<*>{objccomment}		;

"#"{whitespace}(import|include){whitespace}("<"[^>]+">"|"\""[^\"]+"\"") {
			yylval.stringVal = YYSTRING; return IMPORTSTRING;
		}

@implementation			{	BEGIN(CLASS); return IMPLEMENTATION; }

<CLASS>[:()<>,]			{	return yytext[0]; }

    /* In a protocol, @end might be seen while still in METHOD mode */
<CLASS,METHOD>@end		{	BEGIN(INITIAL); return END;	}


{ident}					{	if (YYSTATE != INITIAL)
							{
								yylval.stringVal = YYSTRING; return IDENT;
							}
						}

    /* In a protocol, + or - might be seen while still in METHOD mode */
<CLASS,METHOD>[+-]		{	BEGIN(METHOD); yylval.charVal = yytext[0];	return METHODSTART; }

<METHOD>":"				{	return yytext[0]; }

<METHOD>{statictype}	{	yylval.stringVal = YYSTRING; return STATICTYPE;	}

<METHOD>";"				{	return yytext[0]; }

<CLASS,METHOD>"{"		{	BEGIN(SQUIGGLES); numSquiggles++; }
<SQUIGGLES>{ccomment}	;
<SQUIGGLES>{conststring}	;
<SQUIGGLES>"{"			{	numSquiggles++; }
<SQUIGGLES>"}"			{	numSquiggles--; 
							if (numSquiggles == 0)
							{
								BEGIN(CLASS);
								return SQUIGGLEBLOCK;
							}
						}
<SQUIGGLES>[^{}/"]+		;


{whitespace}				;
.|\n						;


%%

void ADSetSourceFile(FILE *file)
{
	yyin = file;
}

/*
void ResetYYLoc(YYLTYPE *yyLocation, char *text, int textLength)
{
	int i;
	int lastNewlineIndex = 0;

	yyLocation->first_line = yyLocation->last_line;
	yyLocation->first_column = yyLocation->last_column + 1;

	printf("%s\n", text);
	
	for (i=0; i<textLength; i++)
	{
		if (text[i] == '\n')
		{
			yyLocation->last_line++;
			lastNewlineIndex = i;
			yyLocation->last_column = 0;
		}
	}
	
	yyLocation->last_column += (textLength - lastNewlineIndex - 1);
}

void SetupYYLoc(YYLTYPE *yyLocation)
{
	yyLocation->first_line = yyLocation->first_column = 1;
	yyLocation->last_line = yyLocation->last_column = 1;
}
*/